import { LoadedData } from "paperclip";
import { ContentChange } from "paperclip-source-writer";
import { ExprSource, NodeStyleInspection, VirtNodeSource } from "paperclip-utils";
import { AvailableBrowser, VirtualNodeSourceInfo } from "../state";
import { BaseAction } from "./base";
import { InstanceAction } from "./instance-actions";
export declare enum ServerActionType {
    INSTANCE_CHANGED = "INSTANCE_CHANGED",
    CRASHED = "CRASHED",
    ALL_PC_CONTENT_LOADED = "ALL_PC_CONTENT_LOADED",
    REVEAL_EXPRESSION_SOURCE_REQUESTED = "REVEAL_EXPRESSION_SOURCE_REQUESTED",
    INIT_PARAM_DEFINED = "INIT_PARAM_DEFINED",
    BROWSERSTACK_BROWSERS_LOADED = "INIT_PARAM_BROWSERSTACK_BROWSERS_LOADEDDEFINED",
    PC_SOURCE_EDITED = "PC_SOURCE_EDITED",
    VIRTUAL_NODE_SOURCES_LOADED = "VIRTUAL_NODE_SOURCES_LOADED",
    VIRTUAL_NODE_STYLES_INSPECTED = "VIRTUAL_NODE_STYLES_INSPECTED"
}
export declare type InstanceChanged = BaseAction<ServerActionType.INSTANCE_CHANGED, {
    targetPCFileUri: string;
    action: InstanceAction;
}>;
export declare type RevealExpressionSourceRequested = BaseAction<ServerActionType.REVEAL_EXPRESSION_SOURCE_REQUESTED, ExprSource>;
export declare type InitParamsDefined = BaseAction<ServerActionType.INIT_PARAM_DEFINED, {
    readonly: boolean;
    availableBrowsers: AvailableBrowser[];
}>;
export declare type VirtualNodeStylesInspected = BaseAction<ServerActionType.VIRTUAL_NODE_STYLES_INSPECTED, Array<[VirtNodeSource, NodeStyleInspection]>>;
export declare type BrowserstackBrowsersLoaded = BaseAction<ServerActionType.BROWSERSTACK_BROWSERS_LOADED, AvailableBrowser[]>;
export declare type AllPCContentLoaded = BaseAction<ServerActionType.ALL_PC_CONTENT_LOADED, Record<string, LoadedData>>;
export declare type PCSourceEdited = BaseAction<ServerActionType.PC_SOURCE_EDITED, Record<string, ContentChange[]>>;
export declare type Crashed = BaseAction<ServerActionType.CRASHED>;
export declare type VirtualNodeSourcesLoaded = BaseAction<ServerActionType.VIRTUAL_NODE_SOURCES_LOADED, Array<VirtualNodeSourceInfo>>;
export declare const instanceChanged: (payload: {
    targetPCFileUri: string;
    action: InstanceAction;
}) => {
    type: ServerActionType.INSTANCE_CHANGED;
    payload: {
        targetPCFileUri: string;
        action: InstanceAction;
    };
    public: boolean;
};
export declare const initParamsDefined: (payload: {
    readonly: boolean;
    availableBrowsers: AvailableBrowser[];
}) => {
    type: ServerActionType.INIT_PARAM_DEFINED;
    payload: {
        readonly: boolean;
        availableBrowsers: AvailableBrowser[];
    };
    public: boolean;
};
export declare const crashed: (payload: undefined) => {
    type: ServerActionType.CRASHED;
    payload: undefined;
    public: boolean;
};
export declare const allPCContentLoaded: (payload: Record<string, LoadedData>) => {
    type: ServerActionType.ALL_PC_CONTENT_LOADED;
    payload: Record<string, LoadedData>;
    public: boolean;
};
export declare const browserstackBrowsersLoaded: (payload: AvailableBrowser[]) => {
    type: ServerActionType.BROWSERSTACK_BROWSERS_LOADED;
    payload: AvailableBrowser[];
    public: boolean;
};
/**
 * @deprecated use explicit reveal API instead
 */
export declare const revealExpressionSourceRequested: (payload: ExprSource) => {
    type: ServerActionType.REVEAL_EXPRESSION_SOURCE_REQUESTED;
    payload: ExprSource;
    public: boolean;
};
export declare const virtualNodeSourcesLoaded: (payload: VirtualNodeSourceInfo[]) => {
    type: ServerActionType.VIRTUAL_NODE_SOURCES_LOADED;
    payload: VirtualNodeSourceInfo[];
    public: boolean;
};
export declare const pcSourceEdited: (payload: Record<string, ContentChange[]>) => {
    type: ServerActionType.PC_SOURCE_EDITED;
    payload: Record<string, ContentChange[]>;
    public: boolean;
};
export declare const virtualNodeStylesInspected: (payload: [VirtNodeSource, NodeStyleInspection][]) => {
    type: ServerActionType.VIRTUAL_NODE_STYLES_INSPECTED;
    payload: [VirtNodeSource, NodeStyleInspection][];
    public: boolean;
};
export declare type ServerAction = InstanceChanged | RevealExpressionSourceRequested | VirtualNodeStylesInspected | VirtualNodeSourcesLoaded | Crashed | PCSourceEdited | AllPCContentLoaded | InitParamsDefined | BrowserstackBrowsersLoaded;
