import * as ve from "paperclip-designer/src/actions";
import { Project, Result, ShareProjectInfo, User, WorkerState } from "../state";
import { ContentChange } from "paperclip-source-writer";
import { AstEmitted } from "paperclip-utils";
export declare type BaseAction<TType extends ActionType, TPayload = undefined> = {
    type: TType;
    payload: TPayload;
};
export declare enum ActionType {
    PROJECT_HOOK_USED = "PROJECT_HOOK_USED",
    ALL_PROJECTS_HOOK_USED = "ALL_PROJECTS_HOOK_USED",
    PROJECT_FILES_HOOK_USED = "PROJECT_FILES_HOOK_USED",
    DELETE_PROJECT_CONFIRMED = "DELETE_PROJECT_CONFIRMED",
    DOWNLOAD_PROJECT_CLICKED = "DOWNLOAD_PROJECT_CLICKED",
    PROJECT_FILES_LOAD_PROGRESS_CHANGED = "PROJECT_FILES_LOAD_PROGRESS_CHANGED",
    SHARE_PROJECT_REQUEST_STATE_CHANGED = "SHARE_PROJECT_REQUEST_STATE_CHANGED",
    SHARE_MODAL_CLOSED = "SHARE_MODAL_CLOSED",
    SHARE_BUTTON_CLICKED = "SHARE_BUTTON_CLICKED",
    RAW_FILE_UPLOADED = "RAW_FILE_UPLOADED",
    FILES_DROPPED = "FILES_DROPPED",
    NEW_PROJECT_ENTERED = "NEW_PROJECT_ENTERED",
    REMOVE_FILE_CLICKED = "REMOVE_FILE_CLICKED",
    FILE_RENAMED = "FILE_RENAMED",
    PROJECT_RENAMED = "PROJECT_RENAMED",
    REQUEST_CHANGED = "REQUEST_CHANGED",
    ENGINE_LOADED = "ENGINE_LOADED",
    GET_PROJECTS_REQUEST_CHANGED = "GET_PROJECTS_REQUEST_CHANGED",
    GET_PROJECT_REQUEST_CHANGED = "GET_PROJECT_REQUEST_CHANGED",
    GET_PROJECT_FILES_REQUEST_CHANGED = "GET_PROJECT_FILES_REQUEST_CHANGED",
    LOGOUT_BUTTON_CLICKED = "LOGOUT_BUTTON_CLICKED",
    ENGINE_CRASHED = "ENGINE_CRASHED",
    SAVED_PROJECT = "SAVED_PROJECT",
    SAVE_BUTTON_CLICKED = "SAVE_BUTTON_CLICKED",
    LOGGED_OUT = "LOGGED_OUT",
    SESSION_REQUEST_STATE_CHANGED = "SESSION_REQUEST_STATE_CHANGED",
    CODE_EDITOR_TEXT_CHANGED = "CODE_EDITOR_TEXT_CHANGED",
    SLIM_CODE_EDITOR_TEXT_CHANGED = "SLIM_CODE_EDITOR_TEXT_CHANGED",
    WORKER_INITIALIZED = "WORKER_INITIALIZED",
    APP_STATE_DIFFED = "APP_STATE_DIFFED",
    CONTENT_CHANGES_CREATED = "CONTENT_CHANGES_CREATED",
    FILE_ITEM_CLICKED = "FILE_ITEM_CLICKED",
    NEW_FILE_NAME_ENTERED = "NEW_FILE_NAME_ENTERED",
    SYNC_PANELS_CLICKED = "SYNC_PANELS_CLICKED",
    ACCOUNT_CONNECTED = "ACCOUNT_CONNECTED"
}
export declare enum AccountKind {
    Google = "google",
    GitHub = "github"
}
export declare type BaseRequestChanged<TType extends ActionType, TData, TPayload = {}> = BaseAction<TType, {
    result: Result<TData>;
} & TPayload>;
export declare type GetProjectsRequestChanged = BaseRequestChanged<ActionType.GET_PROJECTS_REQUEST_CHANGED, Project[]>;
export declare type ShareModalClosed = BaseAction<ActionType.SHARE_MODAL_CLOSED>;
export declare type GetProjectRequestChanged = BaseRequestChanged<ActionType.GET_PROJECT_REQUEST_CHANGED, Project>;
export declare type DownloadProjectClicked = BaseAction<ActionType.DOWNLOAD_PROJECT_CLICKED>;
export declare type GetProjectFilesRequestChanged = BaseRequestChanged<ActionType.GET_PROJECT_FILES_REQUEST_CHANGED, Record<string, string>, {
    projectId: number;
}>;
export declare type ProjectFilesHookUsed = BaseAction<ActionType.PROJECT_FILES_HOOK_USED, {
    projectId: number;
}>;
export declare type RemoveFileClicked = BaseAction<ActionType.REMOVE_FILE_CLICKED, {
    uri: string;
}>;
export declare type FileRenamed = BaseAction<ActionType.FILE_RENAMED, {
    uri: string;
    newUri: string;
}>;
export declare type ProjectHookUsed = BaseAction<ActionType.PROJECT_HOOK_USED, {
    projectId: number;
}>;
export declare type ProjectFilesLoadProgressChanged = BaseAction<ActionType.PROJECT_FILES_LOAD_PROGRESS_CHANGED, number>;
export declare type RawFileUploaded = BaseAction<ActionType.RAW_FILE_UPLOADED, {
    data: Blob;
    path: string;
}>;
export declare type AllProjectsHookUsed = BaseAction<ActionType.ALL_PROJECTS_HOOK_USED>;
export declare type AccountConnected = BaseAction<ActionType.ACCOUNT_CONNECTED, {
    kind: AccountKind;
    details: any;
}>;
export declare type SavedProject = BaseAction<ActionType.SAVED_PROJECT, Result<boolean>>;
declare type TextEditChange = {
    rangeOffset: number;
    rangeLength: number;
    text: string;
};
export declare type EngineLoaded = BaseAction<ActionType.ENGINE_LOADED>;
export declare type ProjectRenamed = BaseAction<ActionType.PROJECT_RENAMED, {
    projectId: number;
    newName: string;
}>;
export declare type LogoutButtonClicked = BaseAction<ActionType.LOGOUT_BUTTON_CLICKED>;
export declare type DeleteProjectConfirmed = BaseAction<ActionType.DELETE_PROJECT_CONFIRMED, {
    projectId: number;
}>;
export declare type ShareProjectRequestStateChanged = BaseRequestChanged<ActionType.SHARE_PROJECT_REQUEST_STATE_CHANGED, ShareProjectInfo>;
export declare type SaveButtonClicked = BaseAction<ActionType.SAVE_BUTTON_CLICKED, {}>;
export declare type EngineCrashed = BaseAction<ActionType.ENGINE_CRASHED, Error>;
export declare type CodeEditorTextChanged = BaseAction<ActionType.CODE_EDITOR_TEXT_CHANGED, TextEditChange[]>;
export declare type SlimEditorTextChanged = BaseAction<ActionType.SLIM_CODE_EDITOR_TEXT_CHANGED, string>;
export declare type SessionRequestStateChanged = BaseRequestChanged<ActionType.SESSION_REQUEST_STATE_CHANGED, User>;
export declare type WorkerInitialized = BaseAction<ActionType.WORKER_INITIALIZED, {
    state: WorkerState;
}>;
export declare type ShareButtonClicked = BaseAction<ActionType.SHARE_BUTTON_CLICKED>;
export declare type AppStateDiffed = BaseAction<ActionType.APP_STATE_DIFFED, {
    ops: any[];
}>;
export declare type ContentChangesCreated = BaseAction<ActionType.CONTENT_CHANGES_CREATED, {
    changes: Record<string, ContentChange[]>;
}>;
export declare type NewFileNameEntered = BaseAction<ActionType.NEW_FILE_NAME_ENTERED, {
    uri: string;
}>;
export declare type SyncPanelsClicked = BaseAction<ActionType.SYNC_PANELS_CLICKED>;
export declare type FileItemClicked = BaseAction<ActionType.FILE_ITEM_CLICKED, {
    uri: string;
}>;
export declare type NewProjectEntered = BaseAction<ActionType.NEW_PROJECT_ENTERED, {
    name: string;
}>;
export declare type LoggedOut = BaseAction<ActionType.LOGGED_OUT>;
export declare type FilesDropped = BaseAction<ActionType.FILES_DROPPED, FileList>;
export declare const newProjectEntered: (payload: {
    name: string;
}) => {
    type: ActionType.NEW_PROJECT_ENTERED;
    payload: {
        name: string;
    };
};
export declare const shareProjectRequestStateChanged: (payload: {
    result: Result<ShareProjectInfo>;
}) => {
    type: ActionType.SHARE_PROJECT_REQUEST_STATE_CHANGED;
    payload: {
        result: Result<ShareProjectInfo>;
    };
};
export declare const rawFileUploaded: (payload: {
    data: Blob;
    path: string;
}) => {
    type: ActionType.RAW_FILE_UPLOADED;
    payload: {
        data: Blob;
        path: string;
    };
};
export declare const projectFilesLoadProgressChanged: (payload: number) => {
    type: ActionType.PROJECT_FILES_LOAD_PROGRESS_CHANGED;
    payload: number;
};
export declare const filesDropped: (payload: FileList) => {
    type: ActionType.FILES_DROPPED;
    payload: FileList;
};
export declare const downloadProjectClicked: (payload: undefined) => {
    type: ActionType.DOWNLOAD_PROJECT_CLICKED;
    payload: undefined;
};
export declare const accountConnected: (payload: {
    kind: AccountKind;
    details: any;
}) => {
    type: ActionType.ACCOUNT_CONNECTED;
    payload: {
        kind: AccountKind;
        details: any;
    };
};
export declare const logoutButtonClicked: (payload: undefined) => {
    type: ActionType.LOGOUT_BUTTON_CLICKED;
    payload: undefined;
};
export declare const shareButtonClicked: (payload: undefined) => {
    type: ActionType.SHARE_BUTTON_CLICKED;
    payload: undefined;
};
export declare const removeFileClicked: (payload: {
    uri: string;
}) => {
    type: ActionType.REMOVE_FILE_CLICKED;
    payload: {
        uri: string;
    };
};
export declare const fileRenamed: (payload: {
    uri: string;
    newUri: string;
}) => {
    type: ActionType.FILE_RENAMED;
    payload: {
        uri: string;
        newUri: string;
    };
};
export declare const sessionRequestStateChanged: (payload: {
    result: Result<User>;
}) => {
    type: ActionType.SESSION_REQUEST_STATE_CHANGED;
    payload: {
        result: Result<User>;
    };
};
export declare const savedProject: (payload: Result<boolean>) => {
    type: ActionType.SAVED_PROJECT;
    payload: Result<boolean>;
};
export declare const loggedOut: (payload: undefined) => {
    type: ActionType.LOGGED_OUT;
    payload: undefined;
};
export declare const engineLoaded: (payload: undefined) => {
    type: ActionType.ENGINE_LOADED;
    payload: undefined;
};
export declare const engineCrashed: (payload: Error) => {
    type: ActionType.ENGINE_CRASHED;
    payload: Error;
};
export declare const saveButtonClicked: (payload: {}) => {
    type: ActionType.SAVE_BUTTON_CLICKED;
    payload: {};
};
export declare const codeEditorChanged: (payload: TextEditChange[]) => {
    type: ActionType.CODE_EDITOR_TEXT_CHANGED;
    payload: TextEditChange[];
};
export declare const slimCodeEditorChanged: (payload: string) => {
    type: ActionType.SLIM_CODE_EDITOR_TEXT_CHANGED;
    payload: string;
};
export declare const workerInitialized: (payload: {
    state: WorkerState;
}) => {
    type: ActionType.WORKER_INITIALIZED;
    payload: {
        state: WorkerState;
    };
};
export declare const appStateDiffed: (payload: {
    ops: any[];
}) => {
    type: ActionType.APP_STATE_DIFFED;
    payload: {
        ops: any[];
    };
};
export declare const contentChangesCreated: (payload: {
    changes: Record<string, ContentChange[]>;
}) => {
    type: ActionType.CONTENT_CHANGES_CREATED;
    payload: {
        changes: Record<string, ContentChange[]>;
    };
};
export declare const newFileNameEntered: (payload: {
    uri: string;
}) => {
    type: ActionType.NEW_FILE_NAME_ENTERED;
    payload: {
        uri: string;
    };
};
export declare const shareModalClosed: (payload: undefined) => {
    type: ActionType.SHARE_MODAL_CLOSED;
    payload: undefined;
};
export declare const projectRenamed: (payload: {
    projectId: number;
    newName: string;
}) => {
    type: ActionType.PROJECT_RENAMED;
    payload: {
        projectId: number;
        newName: string;
    };
};
export declare const fileItemClicked: (payload: {
    uri: string;
}) => {
    type: ActionType.FILE_ITEM_CLICKED;
    payload: {
        uri: string;
    };
};
export declare const deleteProjectConfirmed: (payload: {
    projectId: number;
}) => {
    type: ActionType.DELETE_PROJECT_CONFIRMED;
    payload: {
        projectId: number;
    };
};
export declare const syncPanelsClicked: (payload: undefined) => {
    type: ActionType.SYNC_PANELS_CLICKED;
    payload: undefined;
};
export declare const getProjectRequestChanged: (payload: {
    result: Result<Project>;
}) => {
    type: ActionType.GET_PROJECT_REQUEST_CHANGED;
    payload: {
        result: Result<Project>;
    };
};
export declare const getProjectsRequestChanged: (payload: {
    result: Result<Project[]>;
}) => {
    type: ActionType.GET_PROJECTS_REQUEST_CHANGED;
    payload: {
        result: Result<Project[]>;
    };
};
export declare const getProjectFilesRequestChanged: (payload: {
    result: Result<Record<string, string>>;
} & {
    projectId: number;
}) => {
    type: ActionType.GET_PROJECT_FILES_REQUEST_CHANGED;
    payload: {
        result: Result<Record<string, string>>;
    } & {
        projectId: number;
    };
};
export declare const projectHookUsed: (payload: {
    projectId: number;
}) => {
    type: ActionType.PROJECT_HOOK_USED;
    payload: {
        projectId: number;
    };
};
export declare const projectFilesHookUsed: (payload: {
    projectId: number;
}) => {
    type: ActionType.PROJECT_FILES_HOOK_USED;
    payload: {
        projectId: number;
    };
};
export declare const allProjectsHookUsed: (payload: undefined) => {
    type: ActionType.ALL_PROJECTS_HOOK_USED;
    payload: undefined;
};
export declare type Action = ve.Action | LogoutButtonClicked | EngineLoaded | AccountConnected | SaveButtonClicked | SlimEditorTextChanged | DeleteProjectConfirmed | ProjectFilesLoadProgressChanged | LoggedOut | SessionRequestStateChanged | GetProjectsRequestChanged | AllProjectsHookUsed | ProjectFilesHookUsed | FilesDropped | ProjectHookUsed | ShareModalClosed | ShareProjectRequestStateChanged | FileRenamed | RemoveFileClicked | GetProjectRequestChanged | AstEmitted | GetProjectFilesRequestChanged | EngineCrashed | AppStateDiffed | DownloadProjectClicked | RawFileUploaded | NewProjectEntered | SavedProject | FileItemClicked | ShareButtonClicked | ProjectRenamed | SyncPanelsClicked | NewFileNameEntered | WorkerInitialized | ContentChangesCreated | CodeEditorTextChanged;
export {};
