/// <reference types="node" />
import * as ve from "paperclip-designer/src/state";
import { SourceLocation } from "paperclip-utils";
import { ContentChange } from "paperclip-source-writer";
export declare type User = {
    avatarUrl?: string;
    email?: string;
    id: number;
};
export declare const MAX_FILE_SIZE: number;
export declare type Result<TData> = {
    data?: TData;
    done?: boolean;
    error?: Error;
};
export declare type Project = {
    id: number;
    name: string;
    owner: boolean;
    mainFilePath?: string;
    files: ProjectFile[];
    updatedAt: string;
};
export declare type ProjectFile = {
    id: number;
    path: string;
    url: string;
};
export declare const APP_LOCATIONS: {
    ROOT: string;
    PROJECTS: string;
    PROJECT: string;
    SHARED_PROJECT: string;
};
export declare const matchesLocationPath: (pathname: string, test: string) => boolean;
export declare const getLocationParams: (pathname: string, test: string) => {
    [key: string]: string;
};
/**
 * State that's also recorded with CRDTs -- needs
 * to be synced with undo / redo stack
 */
export declare type WorkerState = {
    currentProjectId?: number;
    canvasFile: string;
    documents: Record<string, string | Blob>;
};
export declare type ShareProjectInfo = {
    link: string;
};
export declare type AppState = {
    user?: Result<User>;
    currentProject?: Result<Project>;
    playgroundUi: {
        pathname: string;
        query: any;
    };
    currentContentChanges: ContentChange[];
    browserSupported?: boolean;
    progressLoadedPercent?: number;
    highlightLocation?: SourceLocation;
    shareProjectInfo?: Result<ShareProjectInfo>;
    currentCodeFilePath: string;
    allProjects?: Result<Project[]>;
    currentProjectFiles?: Result<Record<string, string | Buffer>>;
    saving?: Result<boolean>;
    hasUnsavedChanges?: boolean;
    compact?: boolean;
    slim?: boolean;
    apiHost: string;
} & ve.AppState;
export declare const INITIAL_STATE: AppState;
export declare const getNewFilePath: (name: string, previousNameOrExt: string) => string;
export declare const getWorkerState: (state: AppState) => WorkerState;
export declare const hasUnsavedChanges: (state: AppState, prevState: AppState) => boolean;
export declare const EDITABLE_MIME_TYPES: string[];
export declare const canUpload: (files: FileList) => boolean;
export declare const canEditFile: (name: string) => boolean;
export declare const canPreviewFile: (name: string) => boolean;
export declare const cleanupPath: (path: string) => string;
